package goods

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
)

// Orders 下单
func (s *GoodsServ) Orders(
	info *model.TaGoodsOrders,
	details []model.TaGoodsOrdersDetail,
	coupons []model.TaGoodsOrdersCoupon) error {

	// 校验下单内容
	if err := s.validOrdersInfo(info); err != nil {
		return err
	}

	// 校验各种金额
	if err := s.validBillCharges(info, details); err != nil {
		return err
	}

	// 保存主订单
	if err := s.dao.SaveOrders(info); err != nil {
		return err
	}

	// 保存订单明细
	if err := s.dao.SaveOrdersDetail(details, info.OrdersId); err != nil {
		return err
	}

	account, err := s.custDAO.GetAccountByCust(info.UserId)
	if err != nil {
		utils.LogError("查询用户账户信息出错: " + err.Error())
		return errors.New("查询用户账户信息出错")
	}

	// 如果是使用优惠券
	if coupons != nil && len(coupons) > 0 {
		// TODO
		// 校验优惠券相关
	} else {
		// 如果是使用城币
		accMoney, _ := strconv.ParseFloat(account.Amount, 64)
		payMoney, _ := strconv.ParseFloat(info.ActualAmount, 64)

		if accMoney < payMoney {
			return errors.New("账户余额不足")
		}
	}

	// 保存优惠券使用记录
	if coupons != nil && len(coupons) > 0 {
		if err := s.dao.SaveOrdersCoupon(coupons, info); err != nil {
			utils.LogError("保存优惠信息出错: " + err.Error())
			return errors.New("保存优惠信息出错")
		}

		// TODO
		// 核销优惠券
	}

	// 如果是城币, 则插入用户账户消费记录
	if info.PayType == models.CONSUME_MONEYCHG {
		if err := s.saveCustomerPayRec(account, info); err != nil {
			return err
		}
	}

	return nil
}

// saveCustomerPayRec 保存账户消费流水
func (s *GoodsServ) saveCustomerPayRec(account *model.TaCustomerAccount, info *model.TaGoodsOrders) error {
	accBill := new(model.TaAccountChange)
	accBill.AccountId = account.AccountId
	accBill.Amount = info.ActualAmount
	accBill.CaseId = info.CaseId
	accBill.ChangeSource = models.ACCSOURCE_ORDERS
	accBill.ChangeType = models.CONSUME_MONEYCHG
	accBill.CustomerId = info.UserId
	accBill.CustomerName = info.UserName
	accBill.FloatType = models.ACCOUNT_SPENT
	accBill.SourceId = info.OrdersId
	accBill.SourceName = ""

	return s.custDAO.InsertAccountRecords(accBill)
}

// validOrdersInfo 校验下单内容
// 没有对金额, 数量的关系进行校验
func (s *GoodsServ) validOrdersInfo(info *model.TaGoodsOrders) error {

	caseID := info.CaseId
	if caseID == "" {
		return errors.New("未指定消费案场")
	}

	areaID := info.AreaId
	if areaID == "" {
		return errors.New("未指定消费区域")
	}

	tableID := info.TableId
	if tableID == "" {
		return errors.New("未指定消费桌位")
	}

	custID := info.UserId
	if custID == "" {
		return errors.New("未指定下单人员")
	}

	// 验证区域
	caseArea, err := s.caseDAO.GetCaseAreaByID(areaID)
	if err != nil {
		utils.LogError("查询案场区域出错: " + err.Error())
		return errors.New("验证案场区域出错")
	}

	info.AreaName = caseArea.AreaName
	info.OrgId = caseArea.OrgId

	// 验证桌位
	table, err := s.caseDAO.GetCaseTableByID(tableID)
	if err != nil {
		utils.LogError("查询案场桌位出错: " + err.Error())
		return errors.New("验证案场桌位出错")
	}

	info.TableNo = table.TableNo

	// 验证人员
	cust, err := s.custDAO.GetCustomerByID(custID)
	if err != nil {
		utils.LogError("查询下单人出错: " + err.Error())
		return errors.New("验证下单人出错")
	}

	if cust.Status != models.STATUS_NORMAL {
		return errors.New("下单人状态不正确, 不能下单")
	}

	// 如果是管理人员
	if cust.UserId != "" {
		fibUsers, err := s.custDAO.GetForbidUserByUserID(cust.UserId)
		if err != nil {
			utils.LogError("查询下单人是否被禁出错: " + err.Error())
			return errors.New("验证下单人出错")
		}

		if fibUsers != nil && len(fibUsers) > 0 {
			for _, u := range fibUsers {
				if u.ForbidType == models.FORBID_ORDER && u.Status == models.STATUS_NORMAL {
					return errors.New("当前下单人被禁止点单")
				}
			}
		}
	}

	return nil
}

// validBillCharges 验证下单各种金额
func (s *GoodsServ) validBillCharges(info *model.TaGoodsOrders, details []model.TaGoodsOrdersDetail) error {
	if details == nil || len(details) == 0 {
		return errors.New("没有发现下单明细")
	}

	caseID := info.CaseId

	orgAmount := float64(0.0)
	for _, item := range details {
		goods, err := s.dao.GetGoodsWithPriceByID(item.GoodsId, item.SpecId, caseID)
		if err != nil {
			utils.LogError("查询商品详情失败: " + err.Error())
			return errors.New("校验商品信息失败")
		}

		if goods.GoodsPrice != item.Price {
			return errors.New("商品金额计算错误")
		}

		// TODO 校验库存
		// 商品暂时无库存

		price, _ := strconv.ParseFloat(item.Price, 64)
		orgAmount += price * float64(item.Number)
	}

	// 校验总金额
	orderAmount, _ := strconv.ParseFloat(info.Amount, 64)

	if orgAmount != orderAmount {
		return errors.New("商品金额计算错误")
	}

	return nil
}
